DROP SCHEMA IF EXISTS sharestuff CASCADE;

BEGIN;

CREATE SCHEMA sharestuff;

CREATE OR REPLACE FUNCTION sharestuff.change_db_setting(setting_name TEXT, settings TEXT[]) RETURNS VOID AS $$
BEGIN
    EXECUTE 'ALTER DATABASE ' || quote_ident(current_database()) || ' SET ' || quote_ident(setting_name) || ' = ' || array_to_string(settings, ',');
END;
$$ LANGUAGE plpgsql;

SELECT sharestuff.change_db_setting('search_path', ARRAY['sharestuff','public','pg_catalog']);

CREATE OR REPLACE FUNCTION sharestuff.array_remove_item_by_value(inp ANYARRAY, el ANYELEMENT) RETURNS anyarray AS $$
    SELECT ARRAY_ACCUM(x.e) FROM UNNEST( $1 ) x(e) WHERE x.e <> $2;
$$ LANGUAGE SQL;

CREATE OR REPLACE FUNCTION sharestuff.lowercase( TEXT ) RETURNS TEXT AS $$
    return lc(shift);
$$ LANGUAGE PLPERLU STRICT IMMUTABLE;

CREATE OR REPLACE FUNCTION sharestuff.xml_escape(str TEXT) RETURNS text AS $$
    SELECT REPLACE(REPLACE(REPLACE($1,
       '&', '&amp;'),
       '<', '&lt;'),
       '>', '&gt;');
$$ LANGUAGE SQL IMMUTABLE;

CREATE OR REPLACE FUNCTION sharestuff.regexp_split_to_array(TEXT, TEXT)
RETURNS TEXT[] AS $$
    return encode_array_literal([split $_[1], $_[0]]);
$$ LANGUAGE PLPERLU STRICT IMMUTABLE;

CREATE OR REPLACE FUNCTION sharestuff.is_json( TEXT ) RETURNS BOOL AS $f$
    use JSON::XS;
    my $json = shift();
    eval { JSON::XS->new->allow_nonref->decode( $json ) };
    return $@ ? 0 : 1;
$f$ LANGUAGE PLPERLU;

CREATE TABLE sharestuff.config_flag (
    name    TEXT    PRIMARY KEY,
    label   TEXT    UNIQUE NOT NULL,
    value   TEXT    CHECK (sharestuff.is_json(value)),
    enabled BOOL    NOT NULL DEFAULT FALSE
);
INSERT INTO sharestuff.config_flag (name, label, value, enabled)
    VALUES ('rating.new_rating_bump.days', 'Number of days during which new ratings for a package received increased weight', '30', TRUE)
;

COMMIT;
